syntax = 'proto3';

import "canvas_generated/messages/events.proto";
import "canvas_generated/messages/plugins.proto";

package canvas;

service PluginRunner {
  rpc HandleEvent (Event) returns (stream EventResponse);

  rpc ReloadPlugins (ReloadPluginsRequest) returns (stream ReloadPluginsResponse);
}
